@@ -1,12 +1,11 @@
 # SPDX-FileCopyrightText: 2024-2025 Univention GmbH
 # SPDX-License-Identifier: AGPL-3.0-only
 
-"""|UDM| module for all |blocklist| objects"""
+"""|UDM| module for all blocklist objects"""
 
 from __future__ import annotations
 
 import univention.admin.blocklist
-import univention.admin.filter
 import univention.admin.handlers
 import univention.admin.localization
 from univention.admin.layout import Group, Tab
