@@ -10,9 +10,6 @@ from typing import TYPE_CHECKING, Any
 import univention.admin.filter
 import univention.admin.handlers
 import univention.admin.handlers.dns.alias
-import univention.admin.handlers.dns.forward_zone
-import univention.admin.handlers.dns.reverse_zone
-import univention.admin.handlers.networks.network
 import univention.admin.localization
 import univention.admin.mapping
 import univention.admin.syntax
