@@ -1,6 +1,3 @@
-# Like what you see? Join us!
-# https://www.univention.com/about-us/careers/vacancies/
-#
 # SPDX-FileCopyrightText: 2004-2025 Univention GmbH
 # SPDX-License-Identifier: AGPL-3.0-only
 
@@ -11,6 +8,7 @@ from collections import OrderedDict
 
 
 class acctFlags:
+    """Samba Account Flags"""
 
     def __init__(self, flagstring=None, flags=None, fallbackflags=None):
         if flags is not None:
@@ -20,7 +18,7 @@ class acctFlags:
             if fallbackflags is not None:
                 self.__flags = OrderedDict(fallbackflags)
                 return
-            flagstring = "[U          ]"
+            flagstring = '[U          ]'
         flags = {}
         flagstring = flagstring[1:-1]
         for letter in flagstring:
@@ -35,13 +33,13 @@ class acctFlags:
         return self.__flags[key]
 
     def decode(self):
-        flagstring = "["
+        flagstring = '['
         for flag, set in self.__flags.items():
             if set:
                 flagstring = flagstring + flag
         while len(flagstring) < 12:
-            flagstring += " "
-        flagstring += "]"
+            flagstring += ' '
+        flagstring += ']'
         return flagstring
 
     def set(self, flag):
